#include <AT89X051.H>
#include "mo3_serial.h"

#define TIMERVALUE		0x1000
#define TEILERMAX		5
#define CNT_OF_SCHRITTE	8
//#define CNT_OF_SCHRITTE	4

const code ablauf[] = {1, 1+4, 4, 4+2, 2, 2+8, 8, 8+1};
//const code ablauf[] = {1, 4, 2, 8};

volatile unsigned char energysave = 1;

volatile unsigned char camera = 0;
volatile unsigned char teiler = 0;

volatile unsigned char steptime = TEILERMAX;

volatile unsigned motor1 = 0, motor2 = 0;

volatile unsigned char cmd = 0;
volatile int cnt = 0;


void check_endsch();


void isr1(void) interrupt TF0_VECTOR
{
	TH0 = (0xFFFF - TIMERVALUE) >> 8;
	TL0 = (0xFFFF - TIMERVALUE) & 0xFF;
	
	teiler = (teiler+1)%steptime;
	if(teiler == 0)
	{
		check_endsch();

		if(cmd == 0 || cnt <= 0)
		{
			cmd = 0;
			cnt = 0;

			if(energysave)
				P1 = 0x00; // motoren abdrehen

			return;
		}
		else if(cmd == 'l') // rotate left
			motor2 = (motor2+1)%CNT_OF_SCHRITTE;
		else if(cmd == 'r') // rotate right
			motor2 = (motor2+CNT_OF_SCHRITTE-1)%CNT_OF_SCHRITTE;
		else if(cmd == 'u') // tilt up
			motor1 = (motor1+CNT_OF_SCHRITTE-1)%CNT_OF_SCHRITTE;
		else if(cmd == 'd') // tilt down
			motor1 = (motor1+1)%CNT_OF_SCHRITTE;
		cnt --;

		P1 = (ablauf[motor1] << 4) | ablauf[motor2];
	}
}

void main(void)
{

	int j = 0, i = 0, tmp = 0;

	serial_init();

	// Timer 0 fuer Counter
	TH0 = (0xFFFF - TIMERVALUE) >> 8;
	TL0 = (0xFFFF - TIMERVALUE) & 0xFF;
	TMOD &= 0xF0;
	TMOD |= 0x01;
	// berlauf
	ET0 = 1;
	TR0 = 1;
	EA = 1;

	echo = 0;

	tmp = 20000; while(tmp--);				
	serial_send_string("type h for help\n");
	
	P3 |= ((1 << 5) | (1 << 4) | (1 << 3) | (1 << 2)); // ENDSCHALTER PULL-UP

	while(1)
	{
		if(rec_ready) // received one message
		{
			if(rec_buffer[0] == 'h') // help
			{
				serial_send_string("-- start of help\n");
				tmp = 2000; while(tmp--);				
				serial_send_string("l# : left\nr# : right\n");
				tmp = 2000; while(tmp--);				
				serial_send_string("u# : up\nd# : down\n");
				tmp = 2000; while(tmp--);				
				serial_send_string(" where # is an ");
				tmp = 2000; while(tmp--);				
				serial_send_string("optinal value\n");
				tmp = 2000; while(tmp--);				
				serial_send_string(" of steps ");
				tmp = 2000; while(tmp--);				
				serial_send_string("(max 4 digits)\n");
				tmp = 2000; while(tmp--);				
				serial_send_string("c  : camera on\n");
				tmp = 2000; while(tmp--);				
				serial_send_string("o  : camera off\n");
				tmp = 2000; while(tmp--);				
				serial_send_string("q  : energysave on\n");
				tmp = 2000; while(tmp--);				
				serial_send_string("p  : energysave off\n");
				tmp = 2000; while(tmp--);				
				serial_send_string("sX : set steptime\n");
				tmp = 2000; while(tmp--);				
				serial_send_string(" where X is one digit\n");
				tmp = 2000; while(tmp--);				
				serial_send_string("x  : stop moving\n");
				tmp = 2000; while(tmp--);				
				serial_send_string("g  : get values\n");
				tmp = 2000; while(tmp--);				
				serial_send_string("-- end of help\n");
			}
			else if(rec_buffer[0] == 'l' || rec_buffer[0] == 'r' || rec_buffer[0] == 'u' || rec_buffer[0] == 'd')
			{
				serial_send_string("start moving\n");
				cmd = rec_buffer[0];
				if((rec_buffer[1] >= '0' && rec_buffer[1] <= '9')) // anzahl angegeben
			   	{
					tmp = rec_buffer[1] - '0';

					if((rec_buffer[2] >= '0' && rec_buffer[2] <= '9')) // zweistellig
					{
						tmp *= 10;
						tmp += rec_buffer[2] - '0';

						if((rec_buffer[3] >= '0' && rec_buffer[3] <= '9')) // dreistellig
						{
							tmp *= 10;
							tmp += rec_buffer[3] - '0';

							if((rec_buffer[4] >= '0' && rec_buffer[4] <= '9')) // vierstellig
							{
								tmp *= 10;
								tmp += rec_buffer[4] - '0';
							}
						}
					}

					cnt = tmp;
				}
				else
					cnt = CNT_OF_SCHRITTE * 5;
			}
			else if(rec_buffer[0] == 'x') // stop
			{
				serial_send_string("stop\n");
				cmd = 0;
				cnt = 0;
			}
			else if(rec_buffer[0] == 'c') // camera on
			{
				serial_send_string("camera on\n");
				camera = 1;
			}
			else if(rec_buffer[0] == 'o') // camera off
			{
				serial_send_string("camera off\n");
				camera = 0;
			}
			else if(rec_buffer[0] == 'q') // energysave on
			{
				serial_send_string("energysave on\n");
				energysave = 1;
			}
			else if(rec_buffer[0] == 'p') // energysave off
			{
				serial_send_string("energysave off\n");
				energysave = 0;
			}
			else if(rec_buffer[0] == 'g') // get steptime
			{
				serial_send_string("stept.=");
				serial_send_char(steptime + '0' - 1);
				serial_send_char('\n');
				serial_send_string("e.save=");
				serial_send_char(energysave + '0');
				serial_send_char('\n');
				serial_send_string("cam=");
				serial_send_char(camera + '0');
				serial_send_char('\n');
			}
			else if(rec_buffer[0] == 's') // set steptime
			{
				if((rec_buffer[1] >= '0' && rec_buffer[1] <= '9')) // anzahl angegeben
				{
					steptime = rec_buffer[1] - '0' + 1;
					serial_send_string("set steptime\n");
				}	
				else
					serial_send_string("invalid steptime\n");
			}
			else
				serial_send_string("wrong command\n");
			rec_ready = 0;
		}
	 
		if(camera)
			P3 |= (1 << 7);
		else
			P3 &= ~(1 << 7);
	}
}

void check_endsch()
{
	if((P3 & (1<<5)) && (P3 & (1<<4)))
	{
		serial_send_string("tilt malfunction\n");
		cmd = 0;
		cnt = 0;
	}
	else if((P3 & (1<<3)) && (P3 & (1<<2)))
	{
		serial_send_string("rotate malfunction\n");
		cmd = 0;
		cnt = 0;
	}
	else if(P3 & (1<<5))
	{
		serial_send_string("end of tilt down\n");
		cmd = 'u';
		cnt = CNT_OF_SCHRITTE;
	}
	else if(P3 & (1<<4))
	{
		serial_send_string("end of tilt up\n");
		cmd = 'd';
		cnt = CNT_OF_SCHRITTE;
	}
	else if(P3 & (1<<3))
	{
		serial_send_string("end of rotate right\n");
		cmd = 'l';
		cnt = CNT_OF_SCHRITTE;
	}
	else if(P3 & (1<<2))
	{
		serial_send_string("end of rotate left\n");
		cmd = 'r';
		cnt = CNT_OF_SCHRITTE;
	}
}


